/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: add_instance.c,v 1.3 2001/12/14 05:32:03 builder Exp $
*/

#include "global.h"

int createInstanceContents(ControlInfo *this) {
    WindowRef window = this->adminmgr;
    ControlHandle value;
    CFStringRef text;
    Size actualSize;
    char buffer[255], password[255], confirm[255];
    char filename[255];
    ControlID id;
    FILE *fp;

    // now handle the manager/admin passwords
    id.signature = 'mngr';
    id.id = 128;
    GetControlByID(window, &id, &value);
    GetControlData(value, 0, kControlEditTextCFStringTag, sizeof(CFStringRef),
        &text, &actualSize);
    CFStringGetCString(text, buffer, 255, CFStringGetSystemEncoding());
    if (!strlen(buffer)) {
        alertUser("The manager login name must be non-empty", NULL);
        return -1;
    }
    id.id = 130;
    GetControlByID(window, &id, &value);
    GetControlData(value, kControlEntireControl, kControlEditTextPasswordTag,
        255, password, &actualSize);
    password[actualSize] = '\0';
    if (!strlen(password)) {
        alertUser("The manager password must be non-empty", NULL);
        return -1;
    }
    id.id = 131;
    GetControlByID(window, &id, &value);
    GetControlData(value, kControlEntireControl, kControlEditTextPasswordTag,
        255, confirm, &actualSize);
    confirm[actualSize] = '\0';
    if (strcmp(password, confirm)) {
        alertUser("Manager password and confirm do not match", NULL);
        return -1;
    }

    // now write - initial manager
    sprintf(filename, "%s/inituser", this->new_instance_home);
    if (!(fp = fopen(filename, "w"))) {
        alertUser(strerror(errno), filename);
        return -1;
    }
    fprintf(fp, "%s:%s\n", buffer, password);
    fclose(fp);

    // make var directory - ok if this fails with EEXIST
    sprintf(filename, "%s/var", this->new_instance_home);
    if (mkdir(filename, 0755) == -1 && errno != EEXIST) {
        alertUser(strerror(errno), filename);
        return -1;
    }

    // make import directory
    sprintf(filename, "%s/import", this->new_instance_home);
    if (mkdir(filename, 0755) == -1 && errno != EEXIST) {
        alertUser(strerror(errno), filename);
        return -1;
    }

    if (this->exit_after_instance_create) exit(0);
    HideWindow(this->adminmgr);
    return 0;
}

// return 0 to cancel, 1 to continue, 2 to continue but not create
int checkNewInstanceHome(char *ih) {
    char filename[255];
    struct stat sb;

    // check the selected dir is OK
    sprintf(filename, "%s/var/Data.fs", ih);
    if (stat(filename, &sb) == -1) {
        // ENOENT can only mean the tail of the path - ih must exist
        if (errno != ENOENT) {
            alertUser(strerror(errno), ih);
            return 0;
        }
    } else {
        if (askUser("The instance home you specified appears to already exist",
                "Is it OK to still use that instance home?")) {
            return 2;
        }
    }
    return 1;
}

int handleNewInstance(ControlInfo *this) {
    WindowRef window = this->newinstance;
    ControlRef control;
    ControlID nameID = {NEWINST_SIGNATURE, NEWINST_NAME};
    ControlID homeID = {NEWINST_SIGNATURE, NEWINST_HOME};
    CFStringRef text;
    Size actualSize;
    char home[255], name[255], config_file[255];
    ControllerConfig old, new;
    int create;

    this->exit_after_instance_create = 0;

    // get the name and home dir
    GetControlByID(window, &nameID, &control);
    GetControlData(control, 0, kControlEditTextCFStringTag, sizeof(CFStringRef),
        &text, &actualSize);
    CFStringGetCString(text, name, 255, CFStringGetSystemEncoding());
    if (!strlen(name)) {
        alertUser("The instance name must be non-empty", NULL);
        return -1;
    }

    GetControlByID(window, &homeID, &control);
    GetControlData(control, 0, kControlEditTextCFStringTag, sizeof(CFStringRef),
        &text, &actualSize);
    CFStringGetCString(text, home, 255, CFStringGetSystemEncoding());
    if (!strlen(home)) {
        alertUser("The instance home must be selected", NULL);
        return -1;
    }
    sprintf(config_file, "%s/%s.conf", this->app_root, name);

    // check the new home dir isn't already occupied
    if (!(create = checkNewInstanceHome(home))) return 0;
    // actual creation of the dir happens once all this is finished...

    // now create the config
    ControllerConfig_init(&old, this->config_file);
    if (ControllerConfig_read(&old, 1) == -1) {
        alertUser("Unable to create new config", "Couldn't read the old config!");
        return 0;
    }
    ControllerConfig_init(&new, config_file);
    if (ControllerConfig_copy(&old, &new) == -1) {
        alertUser("Resource problem", "Unable to create the new config!");
        return 0;
    }
    ControllerConfig_free(&old);

    // and fix up the config
    if (ControllerConfig_set(&new, "environment", "instance_home", home) == -1) {
        alertUser("Resource problem (2)", "Unable to create the new config!");
        return 0;
    }
    if (ControllerConfig_write(&new, NULL) == -1) {
        alertUser(strerror(errno), new.filename);
        ControllerConfig_free(&new);
        return 0;
    }
    ControllerConfig_free(&new);

    // set the controller to use the new config
    this->new_instance_home = strdup(home);
    this->config_name = strdup(name);
    this->config_file = strdup(config_file);

    // add the new instance to the menu
    fillInstancesMenu(this);

    return create;
}


static void handleEditInstances(ControlInfo *this, ControlRef control, MenuRef menu) {
    fprintf(stderr, "Edit instances...\n");
}


static void fillMenuInner(ControlInfo *this, ControlRef control, MenuRef menu) {
    DIR *dir;
    struct dirent *entry;
    char buffer[255];

    // delete the previous menu items
    while (CountMenuItems(menu) > 3) {
        DeleteMenuItem(menu, 1);
    }

    // open the dir and scan
    dir = opendir(this->app_root);
    while (entry = readdir(dir)) {
        // TODO: switch to using glob()?
        if (!strrchr(entry->d_name, '.')) continue;
        if (strcmp(strrchr(entry->d_name, '.'), ".conf") == 0) {
            strcpy(buffer+1, entry->d_name);
            *strrchr(buffer+1, '.') = '\0';
            buffer[0] = strlen(buffer+1);
            InsertMenuItemText(menu, buffer, 0);
        }
    }
    closedir(dir);
    SetControlMaximum(control, CountMenuItems(menu));
}

static void reSelectInstance(ControlInfo *this, ControlRef control, MenuRef menu) {
    int i;
    char buffer[255];

    // re-select the actual current instance
    for (i=1; i < CountMenuItems(menu)-2; i++) {
        GetMenuItemText(menu, i, buffer);
        if (strncmp(this->config_name, buffer+1, buffer[0]) == 0) {
            SetControlValue(control, i);
            break;
        }
    }
}

void fillInstancesMenu(ControlInfo *this) {
    WindowRef window = this->main;
    ControlRef control;
    MenuRef menu;
    ControlID menuID = {CONF_SIG, CONFIG_INSTANCE_NAME};
    // get the menu
    GetControlByID(window, &menuID, &control);
    menu = GetControlPopupMenuRef(control);
    fillMenuInner(this, control, menu);
    reSelectInstance(this, control, menu);
}


void handleChangeInstance(ControlInfo *this) {
    WindowRef window = this->main;
    ControlRef control;
    MenuRef menu;
    ControlID menuID = {CONF_SIG, CONFIG_INSTANCE_NAME};
    int value;
    char buffer[255];

    GetControlByID(window, &menuID, &control);
    menu = GetControlPopupMenuHandle(control);
    value = GetControlValue(control);
    GetMenuItemText(menu, value, buffer);
    buffer[buffer[0]+1] = '\0';
    if (strncmp("New Instance ...", buffer+1, buffer[0]) == 0) {
        reSelectInstance(this, control, menu);
        ShowWindow(this->newinstance);
        SelectWindow(this->newinstance);
        fillMenuInner(this, control, menu);
        // load up the new instance's info into the controller
        setLogFileInfo(this);
        setVersions(this);
        loadConfigIntoWindow(this, this->main);
    } else if (strncmp("Edit Instances ...", buffer+1, buffer[0]) == 0) {
        reSelectInstance(this, control, menu);
        handleEditInstances(this, control, menu);
        fillMenuInner(this, control, menu);
    } else {
        free(this->config_name);
        this->config_name = strdup(buffer+1);
        free(this->config_file);
        sprintf(buffer, "%s/%s.conf", this->app_root, this->config_name);
        this->config_file = strdup(buffer);
        // load up the new instance's info into the controller
        setLogFileInfo(this);
        setVersions(this);
        loadConfigIntoWindow(this, this->main);
    }
}

