/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: alerts.c,v 1.3 2001/09/26 08:41:34 richard Exp $
*/

#include "global.h"
extern config_entry config_entries[];

// the set of commands that we'll be needing to execute with root privileges
// in addition to python
char *privileged_commands[NUM_COMMANDS] = {
    "/bin/kill", "/bin/mv", "/usr/sbin/chown", "/bin/chmod", "/usr/bin/touch"
};


OSStatus initialiseAuthorization(ControlInfo *this) {
    AuthorizationRights rights;
     
    this->authorized = 0;
   
    // We just want the user's current authorization environment,
    // so we aren't asking for any additional rights yet.
    rights.count = 0;
    rights.items = NULL;        
    return AuthorizationCreate(&rights, kAuthorizationEmptyEnvironment, 
                              kAuthorizationFlagDefaults, &this->authorizationRef);
}

static void freeAuthorization(ControlInfo *this) {
    AuthorizationFree(this->authorizationRef, kAuthorizationFlagDestroyRights);
    this->authorized = 0;
}

// figure if we're allowed to exec the program as root
int determineAuthorization(ControlInfo *this, char *program) {
    AuthorizationRights rights;
    AuthorizationItem items[NUM_COMMANDS+1];
    OSStatus err = 0;
    int i;

    // For the right to execute tools as root, kAuthorizationRightExecute,
    // they should hold a pointer to a C string containing the path to 
    // the tool you want to execute, and the length of the C string path.
    items[0].name = kAuthorizationRightExecute;
    items[0].value = program;
    items[0].valueLength = strlen(program);
    items[0].flags = 0;
    for (i=0; i<NUM_COMMANDS; i++) {
        items[i+1].name = kAuthorizationRightExecute;
        items[i+1].value = privileged_commands[i];
        items[i+1].valueLength = strlen(privileged_commands[i]);
        items[i+1].flags = 0;
    }

    // There should be one item in the AuthorizationItems array for each
    // right you want to acquire.
    rights.count = NUM_COMMANDS+1;
    rights.items = items;

    // don't specify kAuthorizationFlagInteractionAllowed and the user won't
    // be prompted - we'll just get yea or nay.
    err = AuthorizationCopyRights(this->authorizationRef, &rights,
        kAuthorizationEmptyEnvironment, kAuthorizationFlagExtendRights, NULL);
    this->authorized = (errAuthorizationSuccess == err);
    return this->authorized;
}


// pass in an authRef initialised by initliaseAuthorization
int authorizationUnlock(ControlInfo *this, char *program) {
    AuthorizationRights rights;
    AuthorizationFlags flags;
    AuthorizationItem items[NUM_COMMANDS+1];
    OSStatus err = 0;
    int i;

    items[0].name = kAuthorizationRightExecute;
    items[0].value = program;
    items[0].valueLength = strlen(program);
    items[0].flags = 0;
    for (i=0; i<NUM_COMMANDS; i++) {
        items[i+1].name = kAuthorizationRightExecute;
        items[i+1].value = privileged_commands[i];
        items[i+1].valueLength = strlen(privileged_commands[i]);
        items[i+1].flags = 0;
    }

    rights.count = NUM_COMMANDS+1;
    rights.items = items;

    // destroy previous rights
    AuthorizationFree(this->authorizationRef, kAuthorizationFlagDestroyRights);

    // let the user be prompted for their password to unlock
    flags = kAuthorizationFlagInteractionAllowed | kAuthorizationFlagExtendRights;
    err = AuthorizationCreate(&rights, kAuthorizationEmptyEnvironment, 
        flags, &this->authorizationRef);
    if (err == errAuthorizationDenied) {
        alertUser("Authorisation denied",
            "You do no have sufficient authorisation to perform this action.");
    }
    this->authorized = (errAuthorizationSuccess == err);
    return this->authorized;
}


// pass in an authRef initialised by initliaseAuthorization
// The args parameter to AuthorizationExecuteWithPrivileges is
// a NULL terminated array of C string pointers, like exec but without
// the program!
OSStatus launchWithPrivileges(ControlInfo *this, char *program, char **args) {
    return AuthorizationExecuteWithPrivileges(this->authorizationRef, program,
        0, args, NULL);
}

// handle the clicking of the lock button
void handleLock(ControlInfo *this) {
    ControllerConfig config;
    char *program;

    // get the program from the config file
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) return;
    program = ControllerConfig_get(&config, "environment", "python");

    // make sure the program's valid
    if (!program) {
        alertUser("Python configuration invalid",
            "You must configure the python interpreter before unlocking");
        ControllerConfig_free(&config);
        return;
    }

    // if currently authorised, nuke it
    if (determineAuthorization(this, program)) {
        // free up existing auth structure if we were previously authorised
        if (this->authorized) {
            freeAuthorization(this);
            initialiseAuthorization(this);
        }
        this->authorized = 0;
    } else {
        // try to unlock
        authorizationUnlock(this, program);
    }
    ControllerConfig_free(&config);

    // set the control icon, text and other button states
    setStatus(this, NULL, this->server_state);
}

// determine if privileges are required to start the server
int isStartPrivilegeRequired(ControllerConfig *config) {
    config_entry *entry;
    int i, port;
    struct stat sb;
    char buffer[255];

    // check the ports
    for (i=0; i < NUM_CONFIG_ENTRIES; i++) {
        entry = &config_entries[i];
        if (strcmp(entry->section, "services")) continue;
        port = atoi(ControllerConfig_get(config, entry->section, entry->name));
        if (port < 1024) return 1;
    }
    
    // check the database files
    sprintf(buffer, "%s/var/Data.fs", ControllerConfig_get(config, "environment",
        "instance_home"));
    if (stat(buffer, &sb) == 0) {
        if (sb.st_uid != getuid()) {
            return 1;
        }
    }

    // check the parent directory too
    *strrchr(buffer, '/') = 0;
    if (stat(buffer, &sb) == 0) {
        if (sb.st_uid != getuid()) {
            return 1;
        }
    }

    // stop checks the server user
    return isLogWritePrivilegeRequired(config) || isStopPrivilegeRequired(config);
}

// determine if privileges are required to stop the server
int isStopPrivilegeRequired(ControllerConfig *config) {
    char *user;
    struct passwd *pwent;

    // check the server user
    user = ControllerConfig_get(config, "environment", "user");
    if (!user) return 0;
    if (!(pwent = getpwnam(user))) {
        return 0;
    }
    
    if (pwent->pw_uid != getuid()) {
        return 1;
    }

    return 0;
}

// determine if privilege is required to save the config
int isConfigWritePrivilegeRequired(ControlInfo *this, ControllerConfig *config) {
    struct stat sb;
    char *parent;

    if (this->server_state & SERVER_RUNNING) {
        // saving config requires server restart, so if we need priv for that, we
        // need priv for this...
        if (isStopPrivilegeRequired(config)) return 1;
        if (isStartPrivilegeRequired(config)) return 1;
    }

    // check the file if it exists
    if (stat(config->filename, &sb) == 0) {
        if (sb.st_uid != getuid()) {
            return 1;
        }
    }

    // check the parent directory too
    parent = strdup(config->filename);
    *strrchr(parent, '/') = 0;
    if (stat(parent, &sb) == 0) {
        if (sb.st_uid != getuid()) {
            free(parent);
            return 1;
        }
    }
    free(parent);
    return 0;
}


// determine if privilege is required to write the log
int isLogWritePrivilegeRequired(ControllerConfig *config) {
    struct stat sb;
    char *logfile, *parent;

    // check the file if it exists
    if (!(logfile = ControllerConfig_get(config, "environment", "logfile"))) {
        return 0;
    }
    if (stat(logfile, &sb) == 0) {
        if (sb.st_uid != getuid()) {
            fprintf(stderr, "file %d, %d\n", sb.st_uid, getuid());
            return 1;
        }
    }

    // check the parent directory too
    parent = strdup(logfile);
    *strrchr(parent, '/') = 0;
    if (stat(parent, &sb) == 0) {
        if (sb.st_uid != getuid()) {
            fprintf(stderr, "parent dir %d, %d\n", sb.st_uid, getuid());
            free(parent);
            return 1;
        }
    }
    free(parent);
    return 0;
}

/* this code is not used any more - well, it barely works with AEWP

// create the logfile as the current user
int authCreateLogfile(ControlInfo *this, ControllerConfig *config) {
    struct stat sb;
    char *logfile, *args[3], buffer[255];
    OSStatus err = noErr;

    // check the file if it exists
    logfile = ControllerConfig_get(config, "environment", "logfile");
    if (stat(logfile, &sb) == -1) {
        // create the file
        args[0] = logfile;
        args[1] = NULL;
        err = AuthorizationExecuteWithPrivileges(this->authorizationRef,
            privileged_commands[COMMAND_TOUCH], 0, args, NULL);
        if (err != noErr) {
            sprintf(buffer, "AEWP returned %ld", err);
            alertUser("Error initialising logfile (touch)", buffer);
            return -1;
        }
    }

    // chown the file
    return authChownLogfile(this, config, 0);
}

// chown the logfile so the server user can write to it
int authChownLogfile(ControlInfo *this, ControllerConfig *config, int server_user) {
    char *logfile, *args[3], buffer[255];
    OSStatus err = noErr;
    struct passwd *pwent;

    logfile = ControllerConfig_get(config, "environment", "logfile");
    if (server_user) {
        args[0] = ControllerConfig_get(config, "environment", "user");
    } else {
        pwent = getpwuid(getuid());
        args[0] = pwent->pw_name;
    }
    args[1] = logfile;
    args[2] = NULL;
    err = AuthorizationExecuteWithPrivileges(this->authorizationRef,
        privileged_commands[COMMAND_CHOWN], 0, args, NULL);
    if (err != noErr) {
        sprintf(buffer, "AEWP returned %ld", err);
        alertUser("Error change logfile owner", buffer);
        return -1;
    }
    return 0;
}
*/
// end
