/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: config.c,v 1.5 2001/12/14 05:32:03 builder Exp $
*/
#include "global.h"

config_entry config_entries[NUM_CONFIG_ENTRIES] = {
    {"environment", "logfile",		{CONF_SIG, CONFIG_LOGFILE},	{CONF_SIG, 0}},
    {"environment", "instance_home",	{CONF_SIG, CONFIG_IH},		{CONF_SIG, 0}},
    {"environment", "software_home",	{CONF_SIG, CONFIG_SH},		{CONF_SIG, 0}},
    {"environment", "python",		{CONF_SIG, CONFIG_PY},		{CONF_SIG, 0}},
    {"environment", "user",		{CONF_SIG, CONFIG_USER},	{CONF_SIG, 0}},

    {"services", "http_port",		{CONF_SIG, CONFIG_HTTP},
                                        {CONF_SIG, CONFIG_HTTP_ON}},
    {"services", "https_port",		{CONF_SIG, CONFIG_HTTPS},
                                        {CONF_SIG, CONFIG_HTTPS_ON}},
    {"services", "webdav_port",		{CONF_SIG, CONFIG_DAV},
                                        {CONF_SIG, CONFIG_DAV_ON}},
    {"services", "webdavs_port",	{CONF_SIG, CONFIG_DAVS},
                                        {CONF_SIG, CONFIG_DAVS_ON}},
    {"services", "ftp_port",		{CONF_SIG, CONFIG_FTP},
                                        {CONF_SIG, CONFIG_FTP_ON}}
};

// handles opening a config file
int ControllerConfig_read(ControllerConfig *config, int whinge) {
    FILE *fp;
    char line[255], *ptr, error[255];
    int line_num=0, i;
    ControllerConfigSection *section = NULL, *new = NULL;
    ControllerConfigEntry *entry = NULL, *new_e = NULL;

    if (!(fp = fopen(config->filename, "rb"))) {
        if (whinge) {
            alertUser(strerror(errno), config->filename);
        }
        return -1;
    }
    while (fgets(line, 255, fp)) {
        line_num++;
        if (strchr("#;", line[0])) continue;
        // match "[foo bar]\n"
        ptr = strchr(line, 0) - 1;
        while (isspace(*ptr) && ptr > line) *ptr-- = 0;
        if (ptr == line) continue;
        if (line[0] == '[' && *ptr == ']') {
            // new section
            new = malloc(sizeof(ControllerConfigSection));
            new->next = NULL;
            new->entries = NULL;
            *ptr = 0;
            new->name = strdup(&line[1]);
            for (i=0; i<strlen(new->name); i++) new->name[i] = tolower(new->name[i]);
            if (section) {
                section->next = new;
            } else {
                config->sections = new;
            }
            section = new;
            entry = NULL;
            continue;
        }
        if (!section) {
            sprintf(error, "Value outside section in config file line %d", line_num);
            alertUser(error, config->filename);
            return -1;
        }
        // "name := value\n"
        new_e = malloc(sizeof(ControllerConfigEntry));
        new_e->next = NULL;
        ptr = line;
        new_e->name = strdup(strsep(&ptr, " \t:="));
        if (!new_e->name || !ptr) {
            sprintf(error, "Not \"name=value\" in config file line %d", line_num);
            alertUser(error, config->filename);
            free(new_e);
            return -1;
        }
        for (i=0; i<strlen(new_e->name); i++) new_e->name[i] = tolower(new_e->name[i]);
        new_e->value = strdup(&ptr[strspn(ptr,  " \t:=")]);
        if (entry) {
            entry->next = new_e;
        } else {
            section->entries = new_e;
        }
        entry = new_e;
    }
    fclose(fp);
    return 0;
}

// save the configuration
int ControllerConfig_write(ControllerConfig *config, AuthorizationRef authorization) {
    FILE *fp;
    ControllerConfigSection *section = NULL;
    ControllerConfigEntry *entry = NULL;
    char *args[3], *tempname, buffer[255], *username;
    OSErr err=noErr;

    // write the config file
    tempname = tmpnam(NULL);
    if (!(fp = fopen(tempname, "wb"))) {
        alertUser(strerror(errno), tempname);
        return -1;
    }
    for (section = config->sections; section; section=section->next) {
        fprintf(fp, "[%s]\n", section->name);
        for (entry = section->entries; entry; entry=entry->next) {
            fprintf(fp, "%s = %s\n", entry->name, entry->value);
        }
    }
    fclose(fp);
    
    // move the tempfile
    if (authorization) {
        // move the file
        args[0] = tempname;
        args[1] = (char *)config->filename;
        args[2] = NULL;
        err = AuthorizationExecuteWithPrivileges(authorization,
            privileged_commands[COMMAND_MOVE], 0, args, NULL);
        if (err != noErr) {
            sprintf(buffer, "AEWP returned %d", err);
            alertUser("Error writing configuration (move)", buffer);
            return -1;
        }

        // chown the file
        if (!(username = ControllerConfig_get(config, "environment", "user"))) {
            alertUser("Gak! Username not set in config?", "Config file not chown'ed to correct user!");
            return -1;
        }
        args[0] = username;
        args[1] = (char *)config->filename;
        args[2] = NULL;
        err = AuthorizationExecuteWithPrivileges(authorization,
            privileged_commands[COMMAND_CHOWN], 0, args, NULL);
        if (err != noErr) {
            sprintf(buffer, "AEWP returned %d", err);
            alertUser("Error writing configuration (chown)", buffer);
            return -1;
        }
    } else {
        // no auth required
        if (rename(tempname, config->filename) == -1) {
            alertUser("Error writing configuration (rename)", strerror(errno));
            return -1;
        }
    }
    return 0;
}

void ControllerConfig_free(ControllerConfig *config) {
    ControllerConfigSection *section = NULL, *next = NULL;
    ControllerConfigEntry *entry = NULL, *next_e = NULL;

    section = config->sections;
    while (section) {
        entry = section->entries;
        while (entry) {
            next_e = entry->next;
            if (entry->name) free(entry->name);
            if (entry->value) free(entry->value);
            free(entry);
            entry = next_e;
        }
        next = section->next;
        if (section->name) free(section->name);
        free(section);
        section = next;
    }
}

void ControllerConfig_init(ControllerConfig *config, const char *filename) {
    config->sections = NULL;
    config->filename = filename;
}

// copy one config to another
int ControllerConfig_copy(ControllerConfig *from, ControllerConfig *to) {
    ControllerConfigSection *section = NULL;
    ControllerConfigEntry *entry = NULL;

    for (section = from->sections; section; section = section->next) {
        for (entry = section->entries; entry; entry = entry->next) {
            if (ControllerConfig_set(to, section->name, entry->name, entry->value)) return -1;
        }
    }
    return 1;
}

// set an item in the config
int ControllerConfig_set(ControllerConfig *config, const char *new_section, const char *new_name,
        const char *new_value) {
    ControllerConfigSection *section = NULL;
    ControllerConfigEntry *entry = NULL;

    for (section = config->sections; section; section = section->next) {
        if (strcmp(section->name, new_section)) continue;
        for (entry = section->entries; entry; entry = entry->next) {
            if (!strcmp(entry->name, new_name)) {
                if (entry->value) free(entry->value);
                entry->value = strdup(new_value);
                return 0;
            }
        }
        break;
    }
    if (!section) {
        section = malloc(sizeof(ControllerConfigSection));
        section->name = strdup(new_section);
        section->entries = NULL;
        section->next = config->sections;
        config->sections = section;
    }
    entry = malloc(sizeof(ControllerConfigEntry));
    entry->name = strdup(new_name);
    entry->value = strdup(new_value);
    entry->next = section->entries;
    section->entries = entry;
    return 0;
}

// get values from the config
char *ControllerConfig_get(ControllerConfig *config, const char *section_name, const char *option) {
    ControllerConfigSection *section = NULL;
    ControllerConfigEntry *entry = NULL;

    for (section = config->sections; section; section = section->next) {
        if (strcmp(section->name, section_name)) continue;
        for (entry = section->entries; entry; entry = entry->next) {
            if (!strcmp(entry->name, option)) {
                return entry->value;
            }
        }
        break;
    }
    return NULL;
}


// remove an entry
int ControllerConfig_remove(ControllerConfig *config, const char *section_name, const char *option) {
    ControllerConfigSection *section = NULL;
    ControllerConfigEntry *entry = NULL, *previous = NULL;

    for (section = config->sections; section; section = section->next) {
        if (strcmp(section->name, section_name)) continue;
        for (entry = section->entries; entry; entry = entry->next) {
            if (!strcmp(entry->name, option)) {
                if (previous) {
                    previous->next = entry->next;
                } else {
                    section->entries = entry->next;
                }
                if (entry->name) free(entry->name);
                if (entry->value) free(entry->value);
                free(entry);
                return 0;
            }
            previous = entry;
        }
        break;
    }
    return -1;
}


int handleSaveConfigure(ControlInfo *this) {
    WindowRef window = this->main;
    ControlHandle value;
    config_entry *entry;
    Size actualSize;
    CFStringRef text;
    char buffer[255], *user;
    int i, ret;
    ControlID id;
    SInt16 cb_value;
    ControllerConfig config;
    struct passwd *pwent;

    ControllerConfig_init(&config, this->config_file);

    // read in the config
    if (ControllerConfig_read(&config, 1) == -1) {
        return -1;
    }

    // update from the UI
    for (i=0; i<NUM_CONFIG_ENTRIES; i++) {
        entry = &config_entries[i];
        id = config_entries[i].cb;
        if (id.id != 0) {
            GetControlByID(window, &id, &value);
            cb_value = GetControlValue(value);
            // if the checkbox is unchecked, nuke the config value
            if (cb_value !=  kControlCheckBoxCheckedValue) {
                // if it fails, it means the entry was already disabled
                ControllerConfig_remove(&config, "services", entry->name);
                continue;
            }
        }
        // now get the actual value
        id = config_entries[i].value;
        GetControlByID(window, &id, &value);
        GetControlData(value, 0, kControlEditTextCFStringTag, sizeof(CFStringRef),
            &text, &actualSize);
        CFStringGetCString(text, buffer, 255, CFStringGetSystemEncoding());
        
        // fix up the config struct
        if (ControllerConfig_set(&config, entry->section, entry->name, buffer) == -1) {
            return -1;
        }
    }

    // if authorised, use it
    if (this->authorized && isConfigWritePrivilegeRequired(this, &config)) {
        ret = ControllerConfig_write(&config, this->authorizationRef);
    } else {
        ret = ControllerConfig_write(&config, NULL);
    }
    if (ret == -1) {
        ControllerConfig_free(&config);
        return -1;
    }

    this->config_valid = 1;

    // set program versions (also checks for their sanity :)
    setVersions(this);
    if (!this->config_valid) return -1;

    // ensure the user is valid (instance_home must be valid to get this far)
    if (!(user = ControllerConfig_get(&config, "environment", "user"))) {
        this->config_valid = 0;
        alertUser("No user specified", "The server user is a required setting");
        ControllerConfig_free(&config);
        return -1;
    }
    if (!(pwent = getpwnam(user))) {
        this->config_valid = 0;
        alertUser("Environment user name not valid", user);
        ControllerConfig_free(&config);
        return -1;
    }
    ControllerConfig_free(&config);

    // possibly do a restart
    if (pingZope(this) & SERVER_RUNNING) {
        if (askUser("Configuration saved.",
                "The server needs to be restarted. Click OK to do so now.")) {
                stopZope(this);
                startZope(this);
        } else {
            alertUser("Configuration saved.",
                "Your new configuration will be used when the server is started next.");
            setStatus(this, "Needs restart", SERVER_NEEDS_RESTART);
        }
    } else {
        alertUser("Configuration saved.",
            "Your new configuration will be used when the server is started next.");
    }

    return 0;
}

pascal void loadConfigIntoWindow(ControlInfo *this, WindowRef window) {
    ControlHandle value;
    CFStringRef text;
    ControllerConfig config;
    ControlID id;
    int i;
    char *config_value;
    config_entry *entry;

    // read in the config
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) {
        return;
    }
    id.signature = CONF_SIG;

    // update from the UI
    for (i=0; i<NUM_CONFIG_ENTRIES; i++) {
        entry = &config_entries[i];
        config_value = ControllerConfig_get(&config, entry->section, entry->name);
        if (config_value == NULL) {
            config_value = "";
        }
        id = config_entries[i].cb;
        if (id.id != 0) {
            GetControlByID(window, &id, &value);
            // do the checkbox
            if (strlen(config_value)) {
                SetControlValue(value, kControlCheckBoxCheckedValue);
            } else {
                SetControlValue(value, kControlCheckBoxUncheckedValue);
            }
            DrawOneControl(value);
        }

        // now get the actual value
        id = config_entries[i].value;
        GetControlByID(window, &id, &value);
        text = CFStringCreateWithCString(NULL, config_value, CFStringGetSystemEncoding());
        SetControlData(value, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text);
        DrawOneControl(value);
    }
    ControllerConfig_free(&config);
}
