/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: control.c,v 1.6 2001/12/17 23:42:05 builder Exp $
*/
#include "global.h"


#ifdef INCLUDE_ENCRYPTION
const int tabList[] = {6, 129, 131, 132, 133, 134};
#else
const int tabList[] = {5, 129, 131, 132, 134};
#endif

pascal OSStatus adminHandler(EventHandlerCallRef handlerRef, EventRef event, void *userData) {
    OSStatus err = eventNotHandledErr;
    HICommand command;
    ControlInfo *this = (ControlInfo *)userData;

    GetEventParameter(event, kEventParamDirectObject, typeHICommand,
        NULL, sizeof(HICommand), NULL, &command);

    switch (command.commandID) {
    case 'psok':
        if (writeAccess(this) == 0) {
            if (pingZope(this)) {
                if (askUser("Admin password saved.",
                        "The server needs to be restarted. Click OK to do so now.")) {
                    stopZope(this);
                    startZope(this);
                } else {
                    alertUser("Admin password saved.",
                        "The new password will be used when the server is started next.");
                }
            } else {
                alertUser("Admin password saved.",
                    "The new password will be used when the server is started next.");
            }
            HideWindow(this->admin);
        }
        err = noErr;
        break;
    case 'pscc':
        HideWindow(this->admin);
        err = noErr;
        break;
    }
    return err;
}

pascal OSStatus managerHandler(EventHandlerCallRef handlerRef, EventRef event, void *userData) {
    OSStatus err = eventNotHandledErr;
    HICommand command;
    ControlInfo *this = (ControlInfo *)userData;

    GetEventParameter(event, kEventParamDirectObject, typeHICommand,
        NULL, sizeof(HICommand), NULL, &command);

    switch (command.commandID) {
    case 'mgok':
//        writeManager(this);
        err = noErr;
        break;
    case 'mgcc':
        HideWindow(this->manager);
        err = noErr;
        break;
    }
    return err;
}

pascal OSStatus newinstanceHandler(EventHandlerCallRef handlerRef, EventRef event, void *userData) {
    OSStatus err = eventNotHandledErr;
    HICommand command;
    ControlInfo *this = (ControlInfo *)userData;
    int create;

    GetEventParameter(event, kEventParamDirectObject, typeHICommand,
        NULL, sizeof(HICommand), NULL, &command);

    switch (command.commandID) {
    case 'nifi':
        handleSelectFolder(this->newinstance, "Select Software Home Folder", NEWINST_SIGNATURE,
            NEWINST_HOME);
        err = noErr;
        break;
    case 'niok':
        create = handleNewInstance(this);
        if (create != -1) {
            HideWindow(this->newinstance);
        }
        // possibly create the instance
        if (create == 1) {
            ShowWindow(this->adminmgr);
            SelectWindow(this->adminmgr);
        }
        err = noErr;
        break;
    case 'nicc':
        HideWindow(this->newinstance);
        err = noErr;
        break;
    }
    return err;
}



void switchTab(ControlInfo *this, int currentTabIndex, ControlRef tabControl) {
    ControlRef userPane;
    ControlRef selectedPane = NULL;
    ControlID controlID;
    int i;

    // seems as good a time as any to load/refresh the config
    loadConfigIntoWindow(this, this->main);

    this->lastTabIndex = currentTabIndex;

    controlID.signature = TAB_SIGNATURE;
    for (i = 1; i < tabList[0] + 1; i++) {
        controlID.id = tabList[i];
        GetControlByID(this->main, &controlID, &userPane);
        if (i == currentTabIndex) {
            selectedPane = userPane;
        } else {
            SetControlVisibility(userPane, false, true);
            DisableControl(userPane);
        }
    }

    if (selectedPane) {
        EnableControl(selectedPane);
        SetControlVisibility(selectedPane, true, true);
    }
    Draw1Control(tabControl);
}

pascal OSStatus tabEventHandler(EventHandlerCallRef handlerRef, EventRef event, void *userData) {
    ControlInfo *this = (ControlInfo *)userData;
    ControlID	controlID;
    ControlRef	tabControl;

    controlID.signature = TAB_SIGNATURE;
    controlID.id = TAB_ID;
    GetControlByID(this->main, &controlID, &tabControl);
    if (GetControlValue(tabControl) == this->lastTabIndex) {
        return eventNotHandledErr;
    }
    switchTab(this, GetControlValue(tabControl), tabControl);

    return noErr;
}

pascal OSStatus handler(EventHandlerCallRef handlerRef, EventRef event,
        void *userData) {
    OSStatus err = eventNotHandledErr;
    HICommand command;
    ControlInfo *this = (ControlInfo *)userData;

    GetEventParameter(event, kEventParamDirectObject, typeHICommand,
        NULL, sizeof(HICommand), NULL, &command);

    switch (command.commandID) {
    case BUTTON_LOCK:
        handleLock(this);
        err = noErr;
        break;
    case BUTTON_START:
        startZope(this);
        err = noErr;
        break;
    case BUTTON_STOP:
        stopZope(this);
        err = noErr;
        break;
    case BUTTON_SAVE_CONFIG:
        handleSaveConfigure(this);
        pingZope(this);
        err = noErr;
        break;
    case BUTTON_ADM_PASS:
        ShowWindow(this->admin);
        SelectWindow(this->admin);
        err = noErr;
        break;
    case BUTTON_ADM_REMOVE:
        deleteAccess(this);
        setStatus(this, NULL, this->server_state);
        err = noErr;
        break;
    case BUTTON_MGMT_SCREEN:
        openManagementScreen(this, TOPLEVEL_MGMT_SCREEN);
        err = noErr;
        break;
    case BUTTON_MGR:
        openManagementScreen(this, "acl_users/manage_users?submit=Add...");
        err = noErr;
        break;
    case BUTTON_ABOUT:
        ShowWindow(this->about);
        SelectWindow(this->about);
        err = noErr;
        break;
#ifdef INCLUDE_ENCRYPTION
    case BUTTON_CONVERT:
        convertKey(this);
        err = noErr;
        break;
#endif
     case BUTTON_FIND_SH:
        handleSelectFolder(this->main, "Select Software Home Folder", CONF_SIG,
            CONFIG_SH);
        err = noErr;
        break;
    case BUTTON_FIND_IH:
        handleSelectFolder(this->main, "Select Instance Home Folder", CONF_SIG,
            CONFIG_IH);
        err = noErr;
        break;
    case BUTTON_FIND_PY:
        handleSelectFile(this->main, "Select Python Interpreter", CONF_SIG,
            CONFIG_PY);
        err = noErr;
        break;
   case BUTTON_FIND_LOGFILE:
        handleSelectPutFile(this->main, "Select Log File", "zopedebug.log", CONF_SIG,
            CONFIG_LOGFILE);
        err = noErr;
        break;
    case BUTTON_FIND_CERT:
        handleSelectFile(this->main, "Select Certificate Export File", CERT_SIGNATURE,
            CERT_SOURCE);
        err = noErr;
        break;
   case BUTTON_FIND_PK:
        handleSelectPutFile(this->main, "Select Save File", "public_key.pem", CERT_SIGNATURE,
            CERT_DEST);
        err = noErr;
        break;
    case BUTTON_VIEW_LOGFILE:
        handleViewLogfile(this);
        err = noErr;
        break;
    case BUTTON_CLEAR_LOG:
        if (askUser("Are you sure?", "Cleared logs are not recoverable.")) {
            fprintf(stderr, "clear log\n");
        }
        err = noErr;
        break;
    case MENU_HELP:
        handleViewHelp(this);
        err = noErr;
        break;
    case BUTTON_CHANGE_INSTANCE:
        handleChangeInstance(this);
        break;
    case BUTTON_QUIT:
        exit(0);
        break;
    }
    return err;
}

void timer(EventLoopTimerRef inTimer, void *userData ) {
    ControlInfo *this = (ControlInfo *)userData;
    if (this->skip) {
        this->skip = 0;
        return;
    }
    pingZope(this);
}


static OSStatus adminmgrHandler(EventHandlerCallRef handlerRef, EventRef event,
        void *userData) {
    HICommand command;

    GetEventParameter(event, kEventParamDirectObject, typeHICommand,
        NULL, sizeof(HICommand), NULL, &command);

    if (command.commandID == 'mgok') {
        createInstanceContents((ControlInfo *)userData);
        return noErr;
    }
    return eventNotHandledErr;
}


static void usage(char *program) {
    printf("Usage: %s [options]\n", program);
    printf(" -n <instance home>: initialise a new instance home in the indicated directory\n");
    printf(" -c <config name>: start up using the indicated config name (default = 'zope')\n");
    exit(1);
}

int main(int argc, char* argv[]) {
    IBNibRef	nibRef;
    OSStatus	err;
    EventTypeSpec commSpec = {kEventClassCommand, kEventCommandProcess};
    EventTypeSpec controlSpec = {kEventClassControl, kEventControlHit};
    ControlInfo this;
    EventLoopTimerRef ref;
    ControlID	controlID;
    ControlRef	tabControl;
    ControllerConfig config;
    char buffer[255];
    extern char *optarg;
    int ch;
    DIR *dir;
    struct dirent *entry;
    struct passwd *pwent;

    // load the NIB
    err = CreateNibReference(CFSTR("main"), &nibRef);
    require_noerr(err, CantGetNibRef);
    err = SetMenuBarFromNib(nibRef, CFSTR("MainMenu"));
    require_noerr(err, CantSetMenuBar);
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), &this.main);
    require_noerr(err, CantCreateWindow);
    err = CreateWindowFromNib(nibRef, CFSTR("AdminPassword"), &this.admin);
    require_noerr(err, CantCreateWindow);
    err = CreateWindowFromNib(nibRef, CFSTR("About"), &this.about);
    require_noerr(err, CantCreateWindow);
    err = CreateWindowFromNib(nibRef, CFSTR("ManagerUser"), &this.manager);
    require_noerr(err, CantCreateWindow);
    err = CreateWindowFromNib(nibRef, CFSTR("AdminMgr"), &this.adminmgr);
    require_noerr(err, CantCreateWindow);
    err = CreateWindowFromNib(nibRef, CFSTR("NewInstance"), &this.newinstance);
    require_noerr(err, CantCreateWindow);
    DisposeNibReference(nibRef);

    // initlialise the auth
    err = initialiseAuthorization(&this);
    require_noerr(err, CantFigureAuth);

    // Set up the window even handlers
    InstallWindowEventHandler(this.main, NewEventHandlerUPP(handler),
        1, &commSpec, (void *)&this, NULL);
    InstallWindowEventHandler(this.admin, NewEventHandlerUPP(adminHandler),
        1, &commSpec, (void *)&this, NULL);
    InstallWindowEventHandler(this.manager, NewEventHandlerUPP(managerHandler),
        1, &commSpec, (void *)&this, NULL);
    InstallWindowEventHandler(this.adminmgr, NewEventHandlerUPP(adminmgrHandler),
        1, &commSpec, (void *)&this, NULL);
    InstallWindowEventHandler(this.newinstance, NewEventHandlerUPP(newinstanceHandler),
        1, &commSpec, (void *)&this, NULL);

    // have a guess at where the app folder is...
    // argv[0], in all my testing, has /blah/blah/Foo.app/Contents/MacOS/Foo
    strcpy(buffer, argv[0]);
    *strrchr(buffer, '/') = 0;
    *strrchr(buffer, '/') = 0;
    *strrchr(buffer, '/') = 0;
    /// ... which should leave us with /blah/blah/Foo.app
    this.app_root = strdup(buffer);

    // find a config name - first in the directory that matches *.conf
    dir = opendir(buffer);
    this.config_name = NULL;
    while (entry = readdir(dir)) {
        // TODO: switch to using glob()?
        if (!strrchr(entry->d_name, '.')) continue;
        if (strcmp(strrchr(entry->d_name, '.'), ".conf") == 0) {
            strcpy(buffer, entry->d_name);
            *strrchr(buffer, '.') = '\0';
            this.config_name = strdup(buffer);
            break;
        }
    }
    closedir(dir);
    // fall back on zope if there's no other takers
    if (!this.config_name) {
        this.config_name = strdup(APP_NAME);
    }

    // handle command-line options
    this.exit_after_instance_create = 0;
    while ((ch = getopt(argc, argv, "phn:c:")) != -1) {
        switch(ch) {
        case 'p':
            // The app gets passed an argument "-psn_0_5636097" or similar for some reason
            // when launched from the Finder. If we find a 'p' in the args, process no more. 
            break;
        case 'n':
            // if we're creating a new instance, do it and exit now
            this.new_instance_home = optarg;
            this.exit_after_instance_create = 1;
            ShowWindow(this.adminmgr);
            RunApplicationEventLoop();
            exit(0);
            break;
        case 'c':
            this.config_name = strdup(optarg);
            break;
        case 'h':
            usage(argv[0]);
            break;
        }
        if (ch == 'p') break;
    }

    // Initialise the ControlInfo
    this.skip = 0;
    this.lastTabIndex = 1;
    this.server_state = SERVER_UNKNOWN;
    sprintf(buffer, "%s/%s.conf", this.app_root, this.config_name);
    this.config_file = strdup(buffer);

    // so try to use that config
    ControllerConfig_init(&config, this.config_file);
    if (ControllerConfig_read(&config, 0) == -1) {
        // the config doesn't exist ... create one
        pwent = getpwuid(getuid());
        ControllerConfig_set(&config, "environment", "user", pwent->pw_name);
        // initalise the environment based on where the controller is installed ...
        // that's as best a guess as we're going to have ...
        sprintf(buffer, "%s/"APP_NAME"/"APP_NAME".log", this.app_root);
        ControllerConfig_set(&config, "environment", "logfile", buffer);
        sprintf(buffer, "%s/"APP_NAME, this.app_root);
        ControllerConfig_set(&config, "environment", "instance_home", buffer);
        ControllerConfig_set(&config, "environment", "software_home", buffer);
        sprintf(buffer, "%s/"APP_NAME"/python", this.app_root);
        ControllerConfig_set(&config, "environment", "python", buffer);

        // default the services
        ControllerConfig_set(&config, "services", "http_port", "80");
        ControllerConfig_set(&config, "services", "https_port", "443");
        ControllerConfig_set(&config, "services", "webdav_port", "81");
        ControllerConfig_set(&config, "services", "webdavs_port", "444");
        ControllerConfig_set(&config, "services", "ftp_port", "21");

        // write the new config
        if (isConfigWritePrivilegeRequired(&this, &config)) {
            // keep on bringing up the unlocker until the user doesn't cancel
            while (determineAuthorization(&this, buffer)) {
                authorizationUnlock(&this, buffer);
            }
            ControllerConfig_write(&config, this.authorizationRef);
        } else {
            ControllerConfig_write(&config, NULL);
        }
        alertUser("New configuration created",
            "Please edit the values in the Environment tab so they make sense!");
        this.config_valid = 0;
    } else {
        this.config_valid = 1;
    }
    ControllerConfig_free(&config);
    setLogFileInfo(&this);

    // set the version strings
    setVersions(&this);

    // Set up the main window tab control
    controlID.signature = TAB_SIGNATURE;
    controlID.id = TAB_ID;
    GetControlByID(this.main, &controlID, &tabControl);
    InstallEventHandler(GetControlEventTarget(tabControl),
        NewEventHandlerUPP(tabEventHandler),
        1, &controlSpec, (void *)&this, NULL);
    switchTab(&this, 1, tabControl);

#ifndef BIZARSHOP_CONTROLLER
    // fill the instances menu
    fillInstancesMenu(&this);
#endif

    // Set up the event timer
    InstallEventLoopTimer(GetCurrentEventLoop(), 0, 10, NewEventLoopTimerUPP(timer),
        (void *)&this, &ref); 

    // Show the main window now
    ShowWindow(this.main);

    // Call the event loop
    RunApplicationEventLoop();

CantFigureAuth:
CantCreateWindow:
CantSetMenuBar:
CantGetNibRef:
    return err;
}

