/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: filechooser.c,v 1.4 2001/12/14 05:32:03 builder Exp $
*/
#include "global.h"

static void handler(NavEventCallbackMessage a, NavCBRecPtr b, void *c) {
}

static char *filechooser(WindowRef window, const char *title) {
    NavDialogCreationOptions options;
    NavReplyRecord reply;
    OSErr anErr = noErr;
    FSRef theFSRef;
    char path[255];
    NavDialogRef dialog;
    CFStringEncoding encoding;
    Size actualSize;
    DescType actualType;

    encoding = CFStringGetSystemEncoding();

    // setup and use the dialog
    NavGetDefaultDialogCreationOptions(&options);
    options.parentWindow = window;
    options.modality = kWindowModalityAppModal;
    options.optionFlags ^= kNavAllowPreviews;
    options.optionFlags ^= kNavAllowMultipleFiles;
    options.windowTitle = CFStringCreateWithCString(NULL, title, encoding);
    NavCreateChooseFileDialog(&options, NULL, NewNavEventUPP(handler), NULL, NULL, NULL, &dialog); 
    NavDialogRun(dialog);
    anErr = NavDialogGetReply(dialog, &reply);
    NavDialogDispose(dialog);

    // handle the result
    if (anErr == noErr && reply.validRecord) {
        anErr = AEGetNthPtr(&(reply.selection), 1, typeFSRef, NULL,
            &actualType, &theFSRef, sizeof(theFSRef), &actualSize);
        if (anErr == noErr) {
            anErr = FSRefMakePath(&theFSRef, path, 255);
            return strdup(path);
        }
        NavDisposeReply(&reply);
        return NULL;
    }
    return NULL;
}

static char *putfilechooser(WindowRef window, const char *title, const char *saved) {
    NavDialogCreationOptions options;
    NavReplyRecord reply;
    OSErr anErr = noErr;
    FSRef fileRefParent;
    AEDesc actualDesc;
    char path[255], name[255], *result=NULL;
    NavDialogRef dialog;
    CFStringEncoding encoding;

    encoding = CFStringGetSystemEncoding();

    // setup and use the dialog
    NavGetDefaultDialogCreationOptions(&options);
    options.parentWindow = window;
    options.modality = kWindowModalityAppModal;
    options.optionFlags ^= kNavAllowPreviews;
    options.optionFlags ^= kNavDontConfirmReplacement;
    options.windowTitle = CFStringCreateWithCString(NULL, title, encoding);
    options.saveFileName = CFStringCreateWithCString(NULL, saved, encoding);
    options.actionButtonLabel = CFSTR("Choose");
    NavCreatePutFileDialog(&options, NULL, NULL, NewNavEventUPP(handler), NULL, &dialog);
    NavDialogRun(dialog);
    anErr = NavDialogGetReply(dialog, &reply);
    NavDialogDispose(dialog);

    // handle the result
    if (anErr == noErr && reply.validRecord) {
        anErr = AECoerceDesc(&reply.selection, typeFSRef, &actualDesc);
        if (anErr == noErr) {
            anErr = AEGetDescData(&actualDesc, &fileRefParent, sizeof(FSRef));
            if (anErr == noErr) {
            	FSRefMakePath(&fileRefParent, path, 255);
                CFStringGetCString(reply.saveFileName, name, 255, encoding);
                result = malloc(strlen(path) + strlen(name) + 2);
                sprintf(result, "%s/%s", path, name);
            }
            AEDisposeDesc(&actualDesc);
        }
        NavDisposeReply(&reply);
    }
    return result;
}


static char *folderchooser(WindowRef window, const char *title) {
    NavDialogCreationOptions options;
    NavReplyRecord reply;
    OSErr anErr = noErr;
    FSRef theFSRef;
    char path[255];
    NavDialogRef dialog;
    CFStringEncoding encoding;
    Size actualSize;
    DescType actualType;

    encoding = CFStringGetSystemEncoding();

    // setup and use the dialog
    NavGetDefaultDialogCreationOptions(&options);
    options.parentWindow = window;
    options.modality = kWindowModalityAppModal;
    options.optionFlags ^= kNavAllowPreviews;
    options.optionFlags ^= kNavAllowMultipleFiles;
    options.windowTitle = CFStringCreateWithCString(NULL, title, encoding);
    NavCreateChooseFolderDialog(&options, NewNavEventUPP(handler), NULL, NULL, &dialog); 
    NavDialogRun(dialog);
    anErr = NavDialogGetReply(dialog, &reply);
    NavDialogDispose(dialog);

    // handle the result
    if (anErr == noErr && reply.validRecord) {
        anErr = AEGetNthPtr(&(reply.selection), 1, typeFSRef, NULL,
            &actualType, &theFSRef, sizeof(theFSRef), &actualSize);
        if (anErr == noErr) {
            anErr = FSRefMakePath(&theFSRef, path, 255);
            return strdup(path);
        }
        NavDisposeReply(&reply);
        return NULL;
    }
    return NULL;
}

char *handleSelectFile(WindowRef window, const char *title, int signature, int id) {
    char *file_selection;
    ControlHandle control;
    CFStringRef text;
    ControlID control_id;

    // choose a file
    if (!(file_selection = filechooser(window, title))) return NULL;

    // set the UI textedit contents
    if (signature) {
        control_id.signature = signature;
        control_id.id = id;
        GetControlByID(window, &control_id, &control);
        text = CFStringCreateWithCString(NULL, file_selection, CFStringGetSystemEncoding());
        SetControlData(control, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text);
        DrawOneControl(control);
    }
    return file_selection;
}

char *handleSelectPutFile(WindowRef window, const char *title, const char *saved,
        int signature, int id) {
    char *file_selection;
    ControlHandle control;
    CFStringRef text;
    ControlID control_id;

    // choose a file
    if (!(file_selection = putfilechooser(window, title, saved))) return NULL;

    // set the UI textedit contents
    if (signature) {
        control_id.signature = signature;
        control_id.id = id;
        GetControlByID(window, &control_id, &control);
        text = CFStringCreateWithCString(NULL, file_selection, CFStringGetSystemEncoding());
        SetControlData(control, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text);
        DrawOneControl(control);
    }
    return file_selection;
}

char *handleSelectFolder(WindowRef window, const char *title, int signature, int id) {
    char *file_selection;
    ControlHandle control;
    CFStringRef text;
    ControlID control_id;

    // choose a file
    if (!(file_selection = folderchooser(window, title))) return NULL;

    // set the UI textedit contents
    if (signature) {
        control_id.signature = signature;
        control_id.id = id;
        GetControlByID(window, &control_id, &control);
        text = CFStringCreateWithCString(NULL, file_selection, CFStringGetSystemEncoding());
        SetControlData(control, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text);
        DrawOneControl(control);
    }
    return file_selection;
}
