/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: global.h,v 1.6 2001/12/17 23:42:05 builder Exp $
*/

#ifndef _CONTROLLER_GLOBAL_H
#define _CONTROLLER_GLOBAL_H
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <pwd.h>
#include <Carbon/Carbon.h>
#include <Security/Authorization.h>
#include <Security/AuthorizationTags.h>

#include "version.h"

#define TAB_ID          128;
#define TAB_SIGNATURE   'tabs'

// strings
#define INFO_STRING 'bzss'
#define INFO_STRING_BIZAR_VERSION	128
#define INFO_STRING_PYTHON_VERSION	129
#define INFO_STRING_ZOPE_VERSION	130
#define INFO_STRING_ABOUT_VERSION	131
#define INFO_STRING_DB_SIZE		132
#define INFO_STRING_STATUS		133
#define INFO_STRING_BIZARSHOP_VERSION	134
#define INFO_STRING_LOCK		136

// buttons
#define BUTTON_SIGNATURE		'bzct'	// for use with IDs to get the controls
#define BUTTON_START			'bzrs'
#define BUTTON_START_ID			128
#define BUTTON_STOP			'bzst'
#define BUTTON_STOP_ID			129
#define BUTTON_SAVE_CONFIG		'bzcf'
#define BUTTON_MGR			'bzmg'
#define BUTTON_MGR_ID			130
#define BUTTON_ADM_PASS			'bzad'
#define BUTTON_ADM_PASS_ID		131
#define BUTTON_ADM_REMOVE		'bzra'
#define BUTTON_ADM_REMOVE_ID		134
#define BUTTON_MGMT_SCREEN		'bzms'
#define BUTTON_MGMT_SCREEN_ID		132
#define BUTTON_ABOUT			'bzab'
#define BUTTON_ENVIRONMENT		'bzse'
#define BUTTON_FIND_SH			'bzsh'
#define BUTTON_FIND_IH			'bzih'
#define BUTTON_PYTHON			'bzsp'
#define BUTTON_FIND_PY			'bzpy'
#define BUTTON_VIEW_LOGFILE		'bzvl'
#define BUTTON_VIEW_LOGFILE_ID		133
#define BUTTON_FIND_LOGFILE		'bzlg'
#define BUTTON_CLEAR_LOG		'bzcl'
#define BUTTON_QUIT			'bzqt'
#define MENU_HELP			'bzch'
#define BUTTON_LOCK			'bzun'
#define BUTTON_LOCK_ID			135
#define BUTTON_CHANGE_INSTANCE		'bzic'
// key convert dialog
#define BUTTON_CONVERT			'ckok'
#define BUTTON_FIND_CERT		'cksk'
#define BUTTON_FIND_PK			'ckdk'
// new instance name dialog
#define BUTTON_NEWINST_FIND		'nifi'
#define BUTTON_NEWINST_OK		'niok'
#define BUTTON_NEWINST_CANCEL		'nicc'


// text entries
#define CONF_SIG	'bzcf'
#define CONFIG_HTTP		130
#define CONFIG_HTTP_ON		131
#define CONFIG_HTTPS		132
#define CONFIG_HTTPS_ON		133
#define CONFIG_DAV		134
#define CONFIG_DAV_ON		135
#define CONFIG_DAVS		136
#define CONFIG_DAVS_ON		137
#define CONFIG_FTP		138
#define CONFIG_FTP_ON		139
#define CONFIG_IH		140
#define CONFIG_SH		141
#define CONFIG_USER		142
#define CONFIG_PY		143
#define CONFIG_LOGFILE		144
#define CONFIG_INSTANCE_NAME	145
#define CONFIG_IH_FIND		146
#define CONFIG_SH_FIND		147
#define CONFIG_LOG_FIND		148
#define CONFIG_PY_FIND		149
#define CONFIG_ENV_SET		150
#define CONFIG_PORTS_SET	151
#define CONFIG_MAX		151

// key convert dialog entries
#define CERT_SIGNATURE		'bzck'
#define CERT_SOURCE		128
#define CERT_PASS		129
#define CERT_DEST		130

// key convert dialog entries
#define NEWINST_SIGNATURE	'bzni'
#define NEWINST_NAME		128
#define NEWINST_HOME		129

// log
#define LOG_SIG		'bzlg'
#define LOG_ID		128

// our understanding of server state
#define SERVER_STOPPED		0x00
#define SERVER_RUNNING		0x01
#define SERVER_LOADING		0x02
#define SERVER_UNKNOWN		0x04
#define SERVER_NEEDS_RESTART	0x09

typedef struct _ControlInfo {
    WindowRef main;
    WindowRef admin;
    WindowRef about;
    WindowRef adminmgr;
    WindowRef manager;
    WindowRef newinstance;
    int	lastTabIndex;
    time_t log_mtime;
    long log_pos;
    int server_state;
    char *app_root;

    // state of the authorization
    AuthorizationRef authorizationRef;
    int authorized;
    
    // config stuff
    char *config_name;
    char *config_file;
    int config_valid;

    // skip the periodic check, once, to avoid some race conditions
    int skip;
    
    // support for new instance stuff
    int exit_after_instance_create;
    char *new_instance_home;
} ControlInfo;

/* add_instance.c */
int createInstanceContents(ControlInfo *this);
void fillInstancesMenu(ControlInfo *this);
void handleChangeInstance(ControlInfo *this);
int handleNewInstance(ControlInfo *this);

/* alerts.h */
void alertUser(const char *message, const char *detail);
int askUser(const char *question, const char *detail);

/* filechooser.c */
char *handleSelectFile(WindowRef window, const char *title, int signature, int id);
char *handleSelectPutFile(WindowRef window, const char *title, const char *saved,
        int signature, int id);
char *handleSelectFolder(WindowRef window, const char *title, int signature, int id);

/* config.c */
typedef struct _ControllerConfigEntry {
    char *name;
    char *value;
    struct _ControllerConfigEntry *next;
} ControllerConfigEntry;

typedef struct _ControllerConfigSection {
    char *name;
    ControllerConfigEntry *entries;
    struct _ControllerConfigSection *next;
} ControllerConfigSection;

typedef struct _ControllerConfig {
    ControllerConfigSection *sections;
    const char *filename;
} ControllerConfig;

// this stuff defines the relationship between config entries and UI
typedef struct _cfg {
    char 	*section;
    char	*name;
    ControlID	value; 
    ControlID	cb;
} config_entry;
#define NUM_CONFIG_ENTRIES 10

int ControllerConfig_read(ControllerConfig *config, int whinge);
int ControllerConfig_write(ControllerConfig *config, AuthorizationRef authorization);
void ControllerConfig_free(ControllerConfig *config);
void ControllerConfig_init(ControllerConfig *config, const char *filename);
char *ControllerConfig_get(ControllerConfig *config, const char *section, const char *label);
int ControllerConfig_remove(ControllerConfig *config, const char *section, const char *label);
int ControllerConfig_copy(ControllerConfig *from, ControllerConfig *to);
int ControllerConfig_set(ControllerConfig *config, const char *section, const char *label, const char *value);
int handleSaveConfigure(ControlInfo *this);
pascal void loadConfigIntoWindow(ControlInfo *this, WindowRef window);

/* status.c */
pascal void setVersions(ControlInfo *this);
void setStatus(ControlInfo *this, char *status_s, int server_status);

/* zope.c */
void setLogFileInfo(ControlInfo *this);
int writeAccess(ControlInfo *this);
void deleteAccess(ControlInfo *this);
int writeManager(ControlInfo *this);
char *escape_command(const char *command);
void startZope(ControlInfo *this);
void stopZope(ControlInfo *this);
int pingZope(ControlInfo *this);
void openManagementScreen(ControlInfo *this, char *url_fragment);
void handleViewLogfile(ControlInfo *this);
void handleViewHelp(ControlInfo *this);

/* authorization.c */
OSStatus initialiseAuthorization(ControlInfo *this);
int determineAuthorization(ControlInfo *this, char *program);
OSStatus launchWithPrivileges(ControlInfo *this, char *program, char **args);
OSStatus launchWithPrivileges2(ControlInfo *this, char *program, char **args);
void handleLock(ControlInfo *this);
int authorizationUnlock(ControlInfo *this, char *program);
#define COMMAND_KILL 0
#define COMMAND_MOVE 1
#define COMMAND_CHOWN 2
#define COMMAND_CHMOD 3
#define COMMAND_TOUCH 4
#define NUM_COMMANDS 5
extern char *privileged_commands[NUM_COMMANDS];
int isStartPrivilegeRequired(ControllerConfig *config);
int isStopPrivilegeRequired(ControllerConfig *config);
int isConfigWritePrivilegeRequired(ControlInfo *this, ControllerConfig *config);
int isLogWritePrivilegeRequired(ControllerConfig *config);
int authCreateLogfile(ControlInfo *this, ControllerConfig *config);
int authChownLogfile(ControlInfo *this, ControllerConfig *config, int server_user);

/* xmlrpc.c */
/*
PyObject *xmlrpc_server(char *server_name_s);
PyObject *xmlrpc_server_auth(char *server_name_s, char *user, char *password);
PyObject *get_page(PyObject *server, char *page_s);
PyObject *add_manager(PyObject *server, char *name, char *password);
*/
#ifdef BIZARSHOP_CONTROLLER
#define APP_NAME "Bizar Shop"
#define TOPLEVEL_MGMT_SCREEN "manage_shops"
#define INCLUDE_ENCRYPTION 1
#else
#define APP_NAME "Zope"
#define TOPLEVEL_MGMT_SCREEN "manage"
#endif

#ifdef INCLUDE_ENCRYPTION
/* encryption.c */
int convertKey(ControlInfo *this);
void handleChooseFile(WindowRef window, NavReplyRecord *reply);
/* pkcs12.h */
int convert_key(char *infile, char *outfile, char *passin);
#endif


//end
#endif
