/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: zope.c,v 1.3 2001/09/26 08:41:34 richard Exp $
*/
#include "global.h"

pascal void setVersions(ControlInfo *this) {
    WindowRef window = this->main;
    ControlHandle control;
    ControlID textID;
    WindowRef about = this->about;
    CFStringRef bzver = CFSTR("Controller Version " VERSION);
    CFStringRef pyver = CFSTR("Python (err)");
    CFStringRef zpver = CFSTR("Zope (err)");
    char buffer[128], *foo, *value, pythoncmd[256], zopever[256];
    FILE *fp;
    ControllerConfig config;
    struct stat dummy;
    int ok=0;
    char *user;
    struct passwd *pwent;

    if (!this->config_valid) return;

    ControllerConfig_init(&config, this->config_file);

    // read in the config
    if (ControllerConfig_read(&config, 1) == -1) {
        return;
    }

    textID.signature = INFO_STRING;

    // controller version
    textID.id = INFO_STRING_BIZAR_VERSION;
    GetControlByID(window, &textID, &control);
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &bzver);
    textID.id = INFO_STRING_ABOUT_VERSION;
    GetControlByID(about, &textID, &control);
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &bzver);

    // server user
    user = ControllerConfig_get(&config, "environment", "user");
    if (!user) return;
    if (!(pwent = getpwnam(user))) {
        alertUser("Invalid environment user name", strerror(errno));
        this->config_valid = 0;
        setStatus(this, NULL, this->server_state);
        return;
    }

    // python version
    textID.id = INFO_STRING_PYTHON_VERSION;
    GetControlByID(window, &textID, &control);
    value = ControllerConfig_get(&config, "environment", "python");
    if (value && strlen(value)) {
        if (stat(value, &dummy) == 0) {
            // try to get the real version
            sprintf(pythoncmd, "\"%s\" -V 2>&1", value);
            if (fp = popen(pythoncmd, "r")) {
                if (fgets(buffer, 128, fp)) {
                    pyver = CFStringCreateWithCString(NULL, buffer, CFStringGetSystemEncoding());
                    ok = 1;
                } else {
                    alertUser("Invalid Python Intepreter", "Didn't respond with valid version");
                }
                pclose(fp);
            } else {
                alertUser("Invalid Python Intepreter", strerror(errno));
                this->config_valid = 0;
                setStatus(this, NULL, this->server_state);
            }
        } else {
            alertUser("Invalid Python Intepreter", strerror(errno));
            this->config_valid = 0;
            setStatus(this, NULL, this->server_state);
        }
    }
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &pyver);

    // zope version
    textID.id = INFO_STRING_ZOPE_VERSION;
    GetControlByID(window, &textID, &control);
    // try to get the real version
    value = ControllerConfig_get(&config, "environment", "software_home");
    if (value && strlen(value)) {
        sprintf(zopever, "%s/lib/python/version.txt", value);
        if (fp = fopen(zopever, "r")) {
            if (fgets(buffer, 128, fp)) {
                if (foo = strchr(buffer, ' ')) {
                    if (foo = strchr(foo + 1, ' ')) {
                        *foo = '\0';
                        zpver = CFStringCreateWithCString(NULL, buffer, CFStringGetSystemEncoding());
                        ok += 2;
                    } else {
                        alertUser("Invalid Software Home", "Version file found is invalid");
                        this->config_valid = 0;
                        setStatus(this, NULL, this->server_state);
                    }
                } else {
                    alertUser("Invalid Software Home", "Version file found is invalid");
                    this->config_valid = 0;
                    setStatus(this, NULL, this->server_state);
                }
            } else {
                alertUser("Invalid Software Home", "Version file found is invalid");
                this->config_valid = 0;
                setStatus(this, NULL, this->server_state);
            }
            fclose(fp);
        } else {
            alertUser("Invalid Software Home", strerror(errno));
            this->config_valid = 0;
            setStatus(this, NULL, this->server_state);
        }
    }
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &zpver);

#ifdef BIZARSHOP_CONTROLLER
    // zope version
    textID.id = INFO_STRING_BIZARSHOP_VERSION;
    GetControlByID(window, &textID, &control);
    // try to get the real version
    value = ControllerConfig_get(&config, "environment", "instance_home");
    if (value && strlen(value)) {
        sprintf(zopever, "%s/Products/BizarShop/version.txt", value);
        if (fp = fopen(zopever, "r")) {
            if (fgets(buffer, 128, fp)) {
                if (foo = strchr(buffer, ' ')) {
                    foo++;
                    foo[strspn(foo, "0123456789.")] = '\0';
                    zpver = CFStringCreateWithCString(NULL, buffer, CFStringGetSystemEncoding());
                    ok += 4;
                } else {
                    alertUser("Invalid BizarShop Installation", "Invalid version file");
                    this->config_valid = 0;
                    setStatus(this, NULL, this->server_state);
                }
            } else {
                alertUser("Invalid BizarShop Installation", "Invalid version file");
                this->config_valid = 0;
                setStatus(this, NULL, this->server_state);
            }
            fclose(fp);
        } else {
            alertUser("Invalid BizarShop Installation", strerror(errno));
            this->config_valid = 0;
            setStatus(this, NULL, this->server_state);
        }
    }
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &zpver);

    if (ok == 7) this->config_valid = 1;
#else
    if (ok == 3) this->config_valid = 1;
#endif

    ControllerConfig_free(&config);
    BeginUpdate(window);
    DrawControls(window);
    EndUpdate(window);
}


void setStatus(ControlInfo *this, char *status_s, int server_state) {
    WindowRef window = this->main;
    ControlHandle control;
    ControlID statusID = {INFO_STRING, INFO_STRING_STATUS};
    ControlID dbID = {INFO_STRING, INFO_STRING_DB_SIZE};
    ControlID startID = {'bzct', BUTTON_START_ID};
    ControlID stopID = {'bzct', BUTTON_STOP_ID};
    ControlID mgmtID = {'bzct', BUTTON_MGMT_SCREEN_ID};
    ControlID mgraddID = {'bzct', BUTTON_MGR_ID};
    ControlID setadmID = {'bzct', BUTTON_ADM_PASS_ID};
    ControlID deleteadmID = {'bzct', BUTTON_ADM_REMOVE_ID};
    ControlID textID = {'bzss', INFO_STRING_LOCK};
    ControlID lockID = {'bzct', BUTTON_LOCK_ID};
    ControlID configID = {'bzcf', 0};
    struct stat sb;
    CFStringRef db_s, lockstr;
    char buffer[128], statusbuf[255], access_file[255];
    ControllerConfig config;
    char db_file[255], *value;
    CFStringRef text;
    ControlButtonContentInfo lock_content;
    IconRef iconRef;
    int authorised, auth_required;

    // store off the state
    this->server_state = server_state;

    if (status_s) {
        GetControlByID(window, &statusID, &control);
        sprintf(statusbuf, "%s server is: %s", this->config_name, status_s);
        text = CFStringCreateWithCString(NULL, statusbuf, CFStringGetSystemEncoding());
        SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &text);
        DrawOneControl(control);
    }

    // get the config, we're gonna need it
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) return;

    // determine what the auth is needed for

    // see if auth is required for anything
    if (!this->config_valid) {
        auth_required = -1;
    } else {
        auth_required = 0;
        if (isStartPrivilegeRequired(&config)) {
            auth_required = 3;
        } else if (isStopPrivilegeRequired(&config)) {
            auth_required = 2;
        }
        if (isConfigWritePrivilegeRequired(this, &config)) {
            auth_required |= 4;
        }
    }
    // actual values that are possible:
    // -1 ack
    // 0 ack
    // 1 poss - stop will be set too
    // 2 nak - if server is running, forces config
    // 3 poss - if server is running, forces config
    // 4 ack
    // 5 nak - stop will be set too
    // 6 ack
    // 7 ack

    // enable/disable the lock
    GetControlByID(window, &lockID, &control);
    if (auth_required < 1) {
        authorised = 0;
        DisableControl(control);
    } else {
        // figure if the user has the appropriate authorization
        value = ControllerConfig_get(&config, "environment", "python");
        if (!determineAuthorization(this, value)) {
            authorised = 0;
        } else {
            authorised = 1;
        }
        EnableControl(control);
    }
    switch (auth_required) {
    case -1:
        GetIconRef(kOnSystemDisk, kSystemIconsCreator, kLockedIcon, &iconRef);
        lockstr = CFSTR("Configuration currently not valid");
        break;
    case 0:
        GetIconRef(kOnSystemDisk, kSystemIconsCreator, kUnlockedIcon, &iconRef);
        lockstr = CFSTR("Authorisation not needed to make changes");
        break;
    case 2:
        if (authorised) lockstr = CFSTR("Click the lock to prevent stopping of the server");
        else lockstr = CFSTR("Click the lock to allow stopping of the server");
        break;
    case 3:
        if (authorised) lockstr = CFSTR("Click the lock to prevent stopping or starting of the server");
        else lockstr = CFSTR("Click the lock to allow stopping or starting of the server");
        break;
    case 4:
        if (authorised) lockstr = CFSTR("Click the lock to prevent configuration changes");
        else lockstr = CFSTR("Click the lock to make configuration changes");
        break;
    case 6:
        if (authorised) lockstr = CFSTR("Click the lock to prevent changes to and stopping of the server");
        else lockstr = CFSTR("Click the lock to allow changes to and stopping of the server");
        break;
    case 7:
        if (authorised) lockstr = CFSTR("Click the lock to prevent changes to and stopping or starting of the server");
        else lockstr = CFSTR("Click the lock to allow changes to and stopping or starting of the server");
        break;
    }

    if (auth_required < 1 || (auth_required > 0 && authorised)) {
        GetIconRef(kOnSystemDisk, kSystemIconsCreator, kUnlockedIcon, &iconRef);
    } else {
        GetIconRef(kOnSystemDisk, kSystemIconsCreator, kLockedIcon, &iconRef);
    }

    // change the lock icon
    lock_content.contentType = kControlContentIconRef;
    lock_content.u.iconRef = iconRef;
    SetControlData(control, 0, kControlRoundButtonContentTag,
        sizeof(ControlButtonContentInfo), (Ptr)&lock_content);
    DrawOneControl(control);

    // and update the text next to the lock too
    GetControlByID(window, &textID, &control);
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &lockstr);
    DrawOneControl(control);

    // to be available in 10.1
    // OverlayApplicationDockTileImage(CGImageRef inImage);
    // RestoreApplicationDockTileImage();

    // disable or enable the config controls
    if (auth_required > 3 && !authorised) {
        // shut it down
        for (configID.id=130; configID.id <= CONFIG_MAX; configID.id++) {
            if (configID.id == 145) continue;
            GetControlByID(window, &configID, &control);
            DisableControl(control);
        }
    } else {
        for (configID.id=130; configID.id <= CONFIG_MAX; configID.id++) {
            if (configID.id == 145) continue;
            GetControlByID(window, &configID, &control);
            EnableControl(control);
        }
    }

    // disable or enable the other controls
    // now disable or enable controls based on server state
    if (server_state & SERVER_RUNNING) {
        // running - disable start button, enable stop button
        GetControlByID(window, &startID, &control);
        DisableControl(control);
        GetControlByID(window, &stopID, &control);
        if (!(auth_required&2) || (auth_required&2 && authorised)) EnableControl(control);
        else DisableControl(control);
    } else if (server_state == SERVER_LOADING || server_state == SERVER_UNKNOWN) {
        // loading or unknown - disable all buttons
        GetControlByID(window, &startID, &control);
        DisableControl(control);
        GetControlByID(window, &stopID, &control);
        DisableControl(control);
    } else {
        // stopped - enable start button, disable stop button
        GetControlByID(window, &startID, &control);
        if (this->config_valid && (!(auth_required&1) || (auth_required&1 && authorised))) {
            EnableControl(control);
        } else {
            // don't enable the start button if we know the startup won't work
            DisableControl(control);
        }
        GetControlByID(window, &stopID, &control);
        DisableControl(control);
    }

    // now disable or enable remaining controls based on server state
    if (server_state == SERVER_RUNNING) {
        GetControlByID(window, &mgmtID, &control);
        EnableControl(control);
        GetControlByID(window, &mgraddID, &control);
        EnableControl(control);
    } else {
        GetControlByID(window, &mgmtID, &control);
        DisableControl(control);
        GetControlByID(window, &mgraddID, &control);
        DisableControl(control);
    }

    // now set the database size info string
    GetControlByID(window, &dbID, &control);

    // figure the database file string
    value = ControllerConfig_get(&config, "environment", "instance_home");
    if (value) {
        sprintf(db_file, "%s/var/Data.fs", value);
        if (stat(db_file, &sb) == 0) {
            sprintf(buffer, "Database Size: %.2fMB", (double)sb.st_size/(1024*1024.0));
            db_s = CFStringCreateWithCString(NULL, buffer, CFStringGetSystemEncoding());
        } else {
            db_s = CFSTR("No Database");
        }
    } else {
        db_s = CFSTR("Database Size: unknown");
    }
    SetControlData(control, 0, kControlStaticTextCFStringTag, sizeof(CFStringRef *), &db_s);

    // set the status of the admin password buttons
    if ((!(auth_required>1) || authorised) && value) {
        sprintf(access_file, "%s/access", value);
        if (stat(access_file, &sb) == 0) {
            // File exists
            GetControlByID(window, &setadmID, &control);
            EnableControl(control);
            GetControlByID(window, &deleteadmID, &control);
            EnableControl(control);
        } else {
            // File doesn't exist
            GetControlByID(window, &setadmID, &control);
            EnableControl(control);
            GetControlByID(window, &deleteadmID, &control);
            DisableControl(control);
        }
    } else {
        // Don't know what the instance home is, or not authorised
        GetControlByID(window, &setadmID, &control);
        DisableControl(control);
        GetControlByID(window, &deleteadmID, &control);
        DisableControl(control);
    }

    ControllerConfig_free(&config);
    BeginUpdate(window);
    DrawControls(window);
    EndUpdate(window);
}
