/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: xmlrpc.c,v 1.3 2001/09/26 08:41:34 richard Exp $
*/
#include <Python.h>

// returns a connection to an xmlrpc server
// transport = xmlrpclib.Transport()
// server = xmlrpclib.Server(server_name_s, transport)
PyObject *xmlrpc_server(char *server_name_s) {
    PyObject *xmlrpc=NULL, *transport=NULL, *server=NULL, *result=NULL;

    xmlrpc = PyImport_ImportModule("xmlrpclib");
    if (xmlrpc == NULL) goto xmlrpc_server_fail;

    transport = PyObject_GetAttrString(xmlrpc, "Transport");
    if (transport == NULL) goto xmlrpc_server_fail;
    transport = PyObject_CallObject(transport, NULL);

    server = PyObject_GetAttrString(xmlrpc, "Server");
    if (server == NULL) goto xmlrpc_server_fail;

    result = PyObject_CallFunction(server, "sO", server_name_s, transport);
xmlrpc_server_fail:
    Py_XDECREF(server);
    Py_XDECREF(xmlrpc);
    Py_XDECREF(transport);
    return result;
}

// returns an authenticated connection to an xmlrpc server
// transport = xmlrpclib.BasicAuthTransport(user, password)
// server = xmlrpclib.Server(server_name_s, transport)
PyObject *xmlrpc_server_auth(char *server_name_s, char *user, char *password) {
    PyObject *xmlrpc=NULL, *transport=NULL, *server=NULL, *result=NULL;

    xmlrpc = PyImport_ImportModule("xmlrpclib");
    if (xmlrpc == NULL) goto xmlrpc_server_auth_fail;

    transport = PyObject_GetAttrString(xmlrpc, "BasicAuthTransport");
    if (transport == NULL) goto xmlrpc_server_auth_fail;
    transport = PyObject_CallFunction(transport, "ss", user, password);

    server = PyObject_GetAttrString(xmlrpc, "Server");
    if (server == NULL) goto xmlrpc_server_auth_fail;

    result = PyObject_CallFunction(server, "sO", server_name_s, transport);
xmlrpc_server_auth_fail:
    Py_XDECREF(server);
    Py_XDECREF(xmlrpc);
    Py_XDECREF(transport);
    return result;
}

// server.index_httml(name, password, ['Manager', 'Owner'], [])
PyObject *get_page(PyObject *server, char *page_s) {
    PyObject *page=NULL, *result=NULL;

    page = PyObject_GetAttrString(server, page_s);
    if (page == NULL) goto get_page_fail;

    result = PyObject_CallObject(page, NULL);
get_page_fail:
    Py_XDECREF(page);
    return result;
}


// server.acl_users.manage_users(name, password, ['Manager', 'Owner'], [])
PyObject *add_manager(PyObject *server, char *name, char *password) {
    PyObject *acl_users=NULL, *adduser=NULL, *result=NULL;

    acl_users = PyObject_GetAttrString(server, "acl_users");
    if (acl_users == NULL) goto add_manager_fail;
    adduser = PyObject_GetAttrString(acl_users, "manage_users");
    if (adduser == NULL) goto add_manager_fail;

    result = PyObject_CallFunction(adduser, "ss[ss][]", name, password, "Manager", "Owner");
add_manager_fail:
    Py_XDECREF(acl_users);
    Py_XDECREF(adduser);
    return result;
}