/*
ZOPE CONTROLLER LICENSE AGREEMENT
---------------------------------

1. This LICENSE AGREEMENT is between Bizar Software Pty Ltd ("Bizar"),
and the Individual or Organization ("Licensee") accessing and otherwise
using Zope Controller software in source or binary form and its associated
documentation.

2. Subject to the terms and conditions of this License Agreement, Bizar
hereby grants Licensee a nonexclusive, royalty-free, world-wide
license to reproduce, analyze, test, perform and/or display publicly,
prepare derivative works, distribute, and otherwise use Zope Controller
alone or in any derivative version, provided, however, that Bizar's
License Agreement and Bizar's notice of copyright, i.e., "Copyright (c)
2001 Bizar Software Pty Ltd; All Rights Reserved" are retained in
Zope Controller alone or in any derivative version prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on
or incorporates Zope Controller or any part thereof, and wants to make
the derivative work available to others as provided herein, then
Licensee hereby agrees to include in any such work a brief summary of
the changes made to Zope Controller.

4. Bizar is making Zope Controller available to Licensee on an "AS IS"
basis.  BIZAR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, BIZAR MAKES NO AND
DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF ZOPE CONTROLLER WILL NOT
INFRINGE ANY THIRD PARTY RIGHTS.

5. BIZAR SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF ZOPE
CONTROLLER FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING ZOPE CONTROLLER,
OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material
breach of its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any
relationship of agency, partnership, or joint venture between Bizar and
Licensee.  This License Agreement does not grant permission to use Bizar
trademarks or trade name in a trademark sense to endorse or promote
products or services of Licensee, or any third party.

8. By copying, installing or otherwise using Zope Controller, Licensee
agrees to be bound by the terms and conditions of this License
Agreement.

$Id: zope.c,v 1.5 2001/12/17 23:42:05 builder Exp $
*/
#include "global.h"
#include <sys/socket.h>
#include <netinet/in.h>

#if !defined(TRAMPOLINE)
# define TRAMPOLINE "/System/Library/CoreServices/AuthorizationTrampoline" /* fallback */
#endif

int writeAccess(ControlInfo *this) {
    WindowRef window = this->admin;
    char *value;
    ControlHandle control;
    Size actualSize;
    char password[255], confirm[255], access_file[255];
    ControlID id;
    FILE *fp;
    ControllerConfig config;
    OSStatus err;

    id.signature = 'pass';
    id.id = 128;
    err = GetControlByID(window, &id, &control);
    err = GetControlData(control, kControlEntireControl, kControlEditTextPasswordTag,
        255, password, &actualSize);
    password[actualSize] = '\0';
    if (!strlen(password)) {
        alertUser("The password must be non-empty", NULL);
        return -1;
    }
    id.id = 129;
    err = GetControlByID(window, &id, &control);
    err = GetControlData(control, kControlEntireControl, kControlEditTextPasswordTag,
        255, confirm, &actualSize);
    confirm[actualSize] = '\0';
    if (strcmp(password, confirm)) {
        alertUser("Password and confirm do not match", NULL);
        return -1;
    }

    // figure the access file string
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) return -1;
    value = ControllerConfig_get(&config, "environment", "instance_home");
    if (!value) {
        alertUser("No Instance Home specified", "I must have an instance home specified,\n"
            "even if it's the same as the Software Home.");
        ControllerConfig_free(&config);
        return -1;
    }
    sprintf(access_file, "%s/access", value);
    ControllerConfig_free(&config);

    // now write
    if (!(fp = fopen(access_file, "w"))) {
        alertUser(strerror(errno), access_file);
        return -1;
    }
    fprintf(fp, "admin:%s\n", password);
    fclose(fp);
    HideWindow(window);
    return 0;
}

void deleteAccess(ControlInfo *this) {
    char *value;
    char access_file[255];
    ControllerConfig config;

    // figure the access file string
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) return;
    value = ControllerConfig_get(&config, "environment", "instance_home");
    if (!value) {
        alertUser("No Instance Home specified", "I must have an instance home specified,\n"
            "even if it's the same as the Software Home.");
        ControllerConfig_free(&config);
        return;
    }
    sprintf(access_file, "%s/access", value);
    ControllerConfig_free(&config);
    if (unlink(access_file) != 0) {
        alertUser(access_file, strerror(errno));
    }    
}

void setLogFileInfo(ControlInfo *this) {
    ControllerConfig config;
    struct stat sb;
    char *logfile;

    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == 0) {
        if (logfile = ControllerConfig_get(&config, "environment", "logfile")) {
            if (stat(logfile, &sb) == 0) {
                this->log_mtime = sb.st_mtime;
                this->log_pos = sb.st_size;
            }
        }
    }
    ControllerConfig_free(&config);
}

/*
checks the logfile and returns:
-1: didn't check at all
 0: checked and all's ok
 1: checked and stuff's bad
*/
int checkLogFile(ControlInfo *this, ControllerConfig *config) {
    char line_s[256], *logfile;
    struct stat sb;
    FILE *fp;
    int problems = -1;

    if (!this->config_valid) return -1;

    // ok, now try getting the logfile
    if (!(logfile = ControllerConfig_get(config, "environment", "logfile"))) {
        return -1;
    }
    if (stat(logfile, &sb) != 0) {
        return -1;
    }
    // make sure something's happened since we last looked
    if (this->log_mtime >= sb.st_mtime) {
        return -1;
    }

    this->log_mtime = sb.st_mtime;
    // yep, there's a logfile and it's been updated recently
    fp = fopen(logfile, "r");
    fseek(fp, this->log_pos, SEEK_SET);
    // pull out lines until we run out
    while (!feof(fp)) {
        if (!fgets(line_s, 256, fp)) break;
        // if we've already run into trouble, skip the checks, otherwise
        // indicate that we've at least parsed some log and all's ok so far
        if (problems > 0) continue;
        else problems = 0;
        if (strstr(line_s, "died on me")) {
            problems = 1;
            alertUser("Server problem",
                "The server has died. Please check the log file");
        } else if (strstr(line_s, "exec failed:")) {
            problems = 1;
            alertUser("Server problem",
                "I was unable to run the server process. Please check the log file.");
        } else if (strstr(line_s, "Address already in use")) {
            problems = 1;
            alertUser("Server problem",
                "Another server is running. Please check the log file.");
        }
    }
    this->log_pos = ftell(fp);
    fclose(fp);

    return problems;
}


int attemptConnect(ControllerConfig *config) {
    int sockfd, result, port=8080;
    struct sockaddr_in address;
    char *http_port;

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd == -1) return -1;

    // try to contact the server by loading the index_html
    if (http_port = ControllerConfig_get(config, "services", "http_port")) {
        port = atoi(http_port);
    }    

    // set up the address
    address.sin_family = AF_INET;
    address.sin_port = port;
    address.sin_addr.s_addr = 0x7f000001;
    result = connect(sockfd, (struct sockaddr *)&address, sizeof(address));
    close(sockfd);
    return result;
}

/*
if no pid file:
    never started
if pid file and it's not valid:
    stopped
if pid file and we can't kill(-1) the process:
    check the log file for messages
    stopped
if loaded:
    running
if can connect to server:
    running
loading...
*/
int pingZope(ControlInfo *this) {
    pid_t processManager;
    char pid_s[128], *ptr;
    FILE *fp;
    char *value;
    ControllerConfig config;
    char processmanager_file[255];

    if (!this->config_valid) {
        setStatus(this, "Unknown", SERVER_UNKNOWN);
        return SERVER_UNKNOWN;
    }

    // figure the processmanager pid file string
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) {
        alertUser("No Instance Home specified in Environment",
            "I must have an instance home specified,\n"
            "even if it's the same as the Software Home.");
        setStatus(this, "Unknown", SERVER_UNKNOWN);
        return SERVER_UNKNOWN;
    }
    value = ControllerConfig_get(&config, "environment", "instance_home");
    if (!value) {
        alertUser("No Instance Home specified in Environment",
            "I must have an instance home specified,\n"
            "even if it's the same as the Software Home.");
        ControllerConfig_free(&config);
        setStatus(this, "Unknown", SERVER_UNKNOWN);
        return SERVER_UNKNOWN;
    }
    sprintf(processmanager_file, "%s/var/zProcessManager.pid", value);

    // Obtain the PID of the process manager
    fp = fopen(processmanager_file, "r");
    if (fp == NULL) {
        if (errno == ENOENT) {
            setStatus(this, "Never started", SERVER_STOPPED);
            return SERVER_STOPPED;
        }
    }
    fgets(pid_s, 128, fp);
    fclose(fp);
    processManager = strtol(pid_s, &ptr, 10);
    if (pid_s == ptr) {
        setStatus(this, "Stopped (bad PID)", SERVER_STOPPED);
        ControllerConfig_free(&config);
        return SERVER_STOPPED;
    }

    // see if the process manager is running
    if (kill(processManager, 0) == -1) {
        // server state unknown - check the logfile so we can pop any alerts
        if (this->server_state == SERVER_UNKNOWN || this->server_state == SERVER_LOADING) {
            checkLogFile(this, &config);
        }
        setStatus(this, "Stopped (no process)", SERVER_STOPPED);
        ControllerConfig_free(&config);
        return SERVER_STOPPED;
    }

    // we're still running but need a restart
    if (this->server_state == SERVER_NEEDS_RESTART) {
        ControllerConfig_free(&config);
        return SERVER_NEEDS_RESTART;
    }

    // we're still running...
    if (this->server_state == SERVER_RUNNING) {
        // set the status anyway
        setStatus(this, "Running", SERVER_RUNNING);
        ControllerConfig_free(&config);
        return SERVER_RUNNING;
    }

    // try to connect to the server to see if loading is finished
    if (attemptConnect(&config) == 0) {
        setStatus(this, "Running", SERVER_RUNNING);
        ControllerConfig_free(&config);
        return SERVER_RUNNING;
    }

    // we're still loading
    setStatus(this, "Loading...", SERVER_LOADING);
    checkLogFile(this, &config);
    ControllerConfig_free(&config);
    return SERVER_LOADING;
}

// hackish escaping of a shell command
char *escape_command(const char *command) {
    char *new, *nptr;

    // worst case - all characters in the command must be escaped
    nptr = new = malloc(strlen(command) * 2);

    while (*command) {
        if (strchr("\\\"' ", *command)) {
            *nptr++ = '\\';
        }
        *nptr++ = *command++;
    }
    *nptr++ = *command++;
    return new;
}

void startZope(ControlInfo *this) {
    ControllerConfig config;
    char *instance_home, *python, *logfile, *software_home,
        *user, *http_port, *webdav_port, *ftp_port, *https_port, *webdavs_port;
    char *args[30];
    int argc, i;
    OSStatus err=noErr;

    this->skip = 1;

    // figure the pid file string
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) return;

    // grab and validate instance_home
    instance_home = ControllerConfig_get(&config, "environment", "instance_home");
    if (!instance_home) {
        alertUser("No Instance Home specified in Environment",
            "I must have an instance home specified!");
        ControllerConfig_free(&config);
        return;
    }

    // grab and validate python
    python = ControllerConfig_get(&config, "environment", "python");
    if (!python) {
        alertUser("No Python Interpreter specified in Environment",
            "I must have a python interpreter specified!");
        ControllerConfig_free(&config);
        return;
    }

    // grab the rest
    logfile = ControllerConfig_get(&config, "environment", "logfile");
    software_home = ControllerConfig_get(&config, "environment", "software_home");
    user = ControllerConfig_get(&config, "environment", "user");
    http_port = ControllerConfig_get(&config, "services", "http_port");
    https_port = ControllerConfig_get(&config, "services", "https_port");
    webdav_port = ControllerConfig_get(&config, "services", "webdav_port");
    webdavs_port = ControllerConfig_get(&config, "services", "webdavs_port");
    ftp_port = ControllerConfig_get(&config, "services", "ftp_port");

    // new, child process - do the setup and then exec
    setenv("INSTANCE_HOME", instance_home, 1);
    if (software_home) {
        chdir(software_home);
    }
    
    // set up the z2s arguments.
    argc = 0;
    args[argc++] = python;       // this argument not needed by ApplicationExecWithPriv
    args[argc++] = "z2s.py";	// TODO: this needs to be configurable!
    if (user) {
        args[argc++] = "-u";
        args[argc++] = user;
    }
    args[argc++] = "-X";
    if (http_port) { args[argc++] = "-w"; args[argc++] = http_port; }
    if (https_port) { args[argc++] = "-y"; args[argc++] = https_port; }
    if (webdav_port) { args[argc++] = "-W"; args[argc++] = webdav_port; }
    if (webdavs_port) { args[argc++] = "-Y"; args[argc++] = webdavs_port; }
    if (ftp_port) { args[argc++] = "-f"; args[argc++] = ftp_port; }
    args[argc] = NULL;

    // final step: redirect output
    if (logfile) {
        setenv("STUPID_LOG_FILE", logfile, 1);
        if (!isLogWritePrivilegeRequired(&config)) {
            // open logfile for writing stdout
            if (!(freopen(logfile, "w", stdout))) {
                alertUser(strerror(errno), logfile);
                return;
            }
            freopen(logfile, "a", stderr);
        }
    }

    // debug
    fprintf(stderr, "Config '%s': '%s'\n", this->config_name, this->config_file);
    fprintf(stderr, "Instance: %s\n", instance_home);
    fprintf(stderr, "Running: %s", python);
    for (i=0; i<argc-1; i++) {
        fprintf(stderr, " %s", args[i]);
    }
    fprintf(stderr, "\n");
    fflush(stderr);

    // launch python / Zope
    if (isStartPrivilegeRequired(&config)) {    
        err = launchWithPrivileges(this, python, &args[1]);
        if (err != noErr) {
            alertUser("Server startup failed", "Check the logfile for errors");
            setStatus(this, "Failed to start", SERVER_STOPPED);
            return;
        }
    } else if (!fork()) {
        execv(python, args);
        fprintf(stderr, "Server startup failed!\nError: %s\n", strerror(errno));
    }

    // we don't need these handles any more...
    ControllerConfig_free(&config);
    setStatus(this, "Loading...", SERVER_LOADING);
    setLogFileInfo(this);
}

void stopZope(ControlInfo *this) {
    pid_t pid;
    char pid_s[255], *pid_start, *pid_end, *ptr;
    FILE *fp;
    int last, priv_required;
    char *value;
    ControllerConfig config;
    char pid_file[255], *args[3], message[255];
    OSStatus err = noErr;
    
    this->skip = 1;
    
    // load the config
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) return;

    if (isStopPrivilegeRequired(&config)) {
        priv_required = 1;
    } else {
        priv_required = 0;
    }
    // figure the pid file string
    value = ControllerConfig_get(&config, "environment", "instance_home");
    if (!value) {
        alertUser("No Instance Home specified in Environment",
            "I must have an instance home specified,\n"
            "even if it's the same as the Software Home.");
        return;
    }
    sprintf(pid_file, "%s/var/Z2.pid", value);
    ControllerConfig_free(&config);

    // Obtain the PID file
    fp = fopen(pid_file, "r");
    if (fp == NULL) {
        if (errno == ENOENT) {
            setStatus(this, "Never started", SERVER_STOPPED);
            return;
        }
    }

    // now loop thru all the PIDs in the file and TERM them
    fgets(pid_s, 255, fp);
    fclose(fp);
    pid_start = pid_s;
    last = 0;
    while (1) {
        pid_end = strchr(pid_start, ' ');
        if (pid_end != NULL) {
            *pid_end = 0;
        } else {
            last = 1;
        }

        // pull out the PID
        pid = strtol(pid_start, &ptr, 10);
        if (pid_start == ptr) {
            setStatus(this, "Indeterminate", SERVER_STOPPED);
            alertUser("Invalid PID in file", pid_file);
            return;
        }
        // kill the process
        if (priv_required) {
            args[0] = pid_start;
            args[1] = NULL;
            errno = 0;
            err = launchWithPrivileges(this, privileged_commands[COMMAND_KILL], args);
            if (err != noErr) {
                sprintf(message, "Error (%s) code %ld", strerror(errno), err);
                alertUser("Unable to kill server", message);
                setStatus(this, "Indeterminate", SERVER_UNKNOWN);
                return;
            }
        } else {
            if (kill(pid, SIGTERM) == -1) {
                alertUser("Unable to kill server", strerror(errno));
                setStatus(this, "Indeterminate", SERVER_UNKNOWN);
                return;
            }
        }

        // finish if that was the last one
        if (last) break;

        // move on to next
        pid_start = pid_end + 1;
    }
    setStatus(this, "Stopped", SERVER_STOPPED);
}

void openManagementScreen(ControlInfo *this, char *url_fragment) {
    ControllerConfig config;
    char *value, url[255];
    ICInstance inst;
    long startSel, endSel, len;
    int err;

    // read in the config
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) {
        return;
    }

    value = ControllerConfig_get(&config, "services", "http_port");
    if (value && strlen(value)) {
        sprintf(url, "http://127.0.0.1:%s/%s", value, url_fragment);
        err = ICStart(&inst, 'BSZH');
        if (err == noErr) {
            startSel = 0;
            endSel = len = strlen(url);
            err = ICLaunchURL(inst, "\p", url, len, &startSel, &endSel);
            ICStop(inst);
            if (err != noErr) {
                alertUser("Unable to open browser with URL", url);
            }
        } else {
            alertUser("Unable to ICStart", url);
        }
    }
    ControllerConfig_free(&config);
}


void handleViewLogfile(ControlInfo *this) {
    ControllerConfig config;
    char *value, command[255];

    // read in the config
    ControllerConfig_init(&config, this->config_file);
    if (ControllerConfig_read(&config, 1) == -1) {
        return;
    }
    value = ControllerConfig_get(&config, "environment", "logfile");
    if (value && strlen(value)) {
        sprintf(command, "open \"%s\"", value);
        system(command);
    } else {
        alertUser("Logfile not set in config", NULL);
    }
    ControllerConfig_free(&config);
}


void handleViewHelp(ControlInfo *this) {
    char command[255];

    // figure the location of the help
    sprintf(command, "open \"%s/Contents/Resources/index.html\"", this->app_root);
    system(command);
}
